@@ -1,8 +1,5 @@
 package Test::CGI::External;
 use 5.006;
-require Exporter;
-@ISA = qw(Exporter);
-@EXPORT_OK = qw//;
 use warnings;
 use strict;
 use utf8;
@@ -13,7 +10,7 @@ use File::Temp 'tempfile';
 use FindBin '$Bin';
 use Test::Builder;
 
-our $VERSION = '0.18';
+our $VERSION = '0.19';
 
 sub new
 {
@@ -21,7 +18,7 @@ sub new
 
     my $tb = Test::Builder->new ();
     $tester{tb} = $tb;
-    $tester{html_validator} = '/home/ben/bin/validate';
+#    $tester{html_validator} = '/home/ben/bin/validate';
 
     return bless \%tester;
 }
@@ -644,9 +641,84 @@ sub set_no_check_content
     $self->{no_check_content} = $value;
 }
 
+sub test_not_implemented
+{
+    my ($self, $method) = @_;
+    my %options;
+    if ($method) {
+	$options{REQUEST_METHOD} = $method;
+    }
+    else {
+	$options{REQUEST_METHOD} = 'GOBBLEDIGOOK';
+    }
+    $options{no_check_request_method} = 1;
+    my $saved_no_check_content = $self->{no_check_content};
+    $self->{no_check_content} = 1;
+    $self->{run_options} = \%options;
+    run_private ($self);
+    #print $options{output}, "\n";
+    $self->check_headers_private ();
+    my $headers = $options{headers};
+    $self->{tb}->ok ($headers->{status}, "Got status header");
+    $self->{tb}->like ($headers->{status}, qr/501/, "Got not implemented status");
+    $self->{no_check_content} = $saved_no_check_content;
+    $self->clear_env ();
+}
+
+sub test_method_not_allowed
+{
+    my ($self, $bad_method) = @_;
+    my $tb = $self->{tb};
+    my %options;
+    $options{REQUEST_METHOD} = $bad_method;
+    $options{no_check_request_method} = 1;
+    my $saved_no_check_content = $self->{no_check_content};
+    $self->{no_check_content} = 1;
+    $self->{run_options} = \%options;
+    run_private ($self);
+    $self->check_headers_private ();
+    my $headers = $options{headers};
+    $tb->ok ($headers->{allow}, "Got Allow header");
+    $tb->like ($headers->{status}, qr/405/, "Got method not allowed status");
+    $self->clear_env ();
+    my @allow = split /,\s*/, $headers->{allow};
+    for my $ok_method (@allow) {
+	# Run the program with each of the headers we were told were
+	# allowed, and see whether the program executes correctly.
+	my %op2;
+	$op2{REQUEST_METHOD} = $ok_method;
+	$self->{run_options} = \%op2;
+	run_private ($self);
+	$self->check_headers_private ();
+	my $headers2 = $op2{headers};
+	# Check that either there is no status line (defaults to 200),
+	# or that there is a status line, and it has status 200.
+	$tb->ok (! $headers2->{status} || $headers2->{status} =~ /200/,
+		 "Method $ok_method specified by Allow: header was allowed");
+	$self->clear_env ();
+    }
+    $self->{no_check_content} = $saved_no_check_content;
+}
+
+# Clear all the environment variables we have set ourselves.
+
+sub clear_env
+{
+my ($self) = @_;
+    for my $e (@{$self->{set_env}}) {
+#        print "Deleting environment variable $e\n";
+        $ENV{$e} = undef;
+    }
+    $self->{set_env} = undef;
+}
+
 sub run
 {
     my ($self, $options) = @_;
+    if (ref $options ne 'HASH') {
+	carp "Use a hash reference as argument, \$tester->run (\\\%options);";
+	return;
+    }
     my $verbose = $self->{verbose};
     local $Test::Builder::Level = $Test::Builder::Level + 1;
     if (! $self->{cgi_executable}) {
@@ -697,6 +769,17 @@ sub run
 	}
     }
 
+    if ($options->{png}) {
+	if ($options->{html} || $options->{json}) {
+	    carp "Contradictory options png and json/html";
+	}
+    }
+    elsif ($options->{html}) {
+	if ($options->{json}) {
+	    carp "Contradictory options json and html";
+	}
+    }
+
 #    eval {
     run_private ($self);
     my $output = $self->{run_options}->{output};
@@ -746,11 +829,7 @@ sub run
     if ($options->{png}) {
 	validate_png ($self);
     }
-    for my $e (@{$self->{set_env}}) {
-#        print "Deleting environment variable $e\n";
-        $ENV{$e} = undef;
-    }
-    $self->{set_env} = undef;
+    $self->clear_env ();
 }
 
 sub tidy_files
